package cz.upol.scripting.game;

/**
 * Element herniho sveta obdelnikoveho tvaru.
 * Kazdy element je identifikovan svym id a tridou.
 * Souradnice jsou dany pravym hornim rohem, vyskou a sirkou
 */
public class Element {
	
	private final String id;
	private final String elementClass;
	
	/** souradnice jednotlivych rohu */
	private Rect rectBounds;
	
	/** rychlost, kterou se objekt pohybuje */
	private Vector speed;
	
	/** priznak, ze objekt byl odstranen a mizi z herniho sveta */
	private boolean disappearing = false;
	
	/**
	 * Vytvori element herniho sveta obdelnikoveho tvaru.
	 * Kazdy element je identifikovan svym id a tridou.
	 * Souradnice jsou dany pravym hornim rohem, vyskou a sirkou.
	 */
	public Element(String id, String elementClass, double x, double y, double width, double height) {
		super();
		this.id = id;
		this.rectBounds = new Rect(x, y, width, height);
		this.speed = new Vector(0, 0);
		this.elementClass = elementClass;
	}

	public String getId() {
		return id;
	}
	
	public Rect getRectBounds() {
		return rectBounds;
	}

	public double getWidth() {
		return rectBounds.getWidth();
	}

	public double getHeight() {
		return rectBounds.getHeight();
	}

	public double getX() {
		return rectBounds.getX();
	}

	public double getY() {
		return rectBounds.getY();
	}

	/**
	 * Vraci rychlost (resp. smer), kterym se element pohybuje
	 */
	public Vector getSpeed() {
		return speed;
	}

	/**
	 * Nastavi rychlost (resp. smer), kterym se element pohybuje
	 */
	public void setSpeed(Vector speed) {
		this.speed = speed;
	} 

	/**
	 * Posune objekt ve smeru dane jeho rychlosti, viz {@link #getSpeed()}
	 */
	public void move() { 
		this.rectBounds = rectBounds.move(speed);
	}
	
	/**
	 * Posune objekt horizontalne dle jeho rychlosti, viz {@link #getSpeed()}
	 */
	public void moveHorizontally() { 
		this.rectBounds = rectBounds.move(new Vector(speed.getX(), 0));
	}
	
	/**
	 * Posune objekt vertikalne dle jeho rychlosti, viz {@link #getSpeed()}
	 */
	public void moveVertically() { 
		this.rectBounds = rectBounds.move(new Vector(0, speed.getY()));
	}
	
	/**
	 * Nastavi souradnice leveho horniho rohu
	 */
	public void setCoords(double x, double y) {
		this.rectBounds = new Rect(x, y, getWidth(), getHeight());
	}

	/**
	 * Nastavi priznak, ze element bude odstranen z herniho sveta
	 */
	public void makeDisappear() {
		disappearing = true;
	}
	
	/**
	 * @return true, pokud je element urcen k odstraneni z herniho sveta
	 */
	public boolean isDisappering() {
		return disappearing;
	}
	
	/**
	 * @return true, pokud se element prekryva s elementem <tt>e</tt>  
	 */
	public boolean isOverlapping(Element e) {
		return this.rectBounds.isOverlapping(e.rectBounds);
	}

	public String getElementClass() {
		return elementClass;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Element other = (Element) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
}
